/*****************************************************************************/
/*!	\file 		IEC61850Types.h
 *	\brief 	IEC 61580 API definition file
 *	\par 		SystemCORP Embedded Technology Pty Ltd
 *
 *			Suite 4/12, Brodie Hall Drive,
 *			Technology Park,
 *			Bentley,
 *			Perth, WA 6102,
 *			Australia.
 *
 *			Phone	: +61 (8) 9472 8500 
 *			Fax		: +61 (8) 9472 9500
 *			Email	: info@systemcorp.com.au
 */
/*****************************************************************************/

#ifndef IEC61580TYPE_H
#define IEC61580TYPE_H		1

#include "sysctype.h"

#ifdef __cplusplus
extern "C" {
#endif

/*! \enum IEC61850_ErrorCodes
 * 	\brief List of error code returned by API functions
 */
enum IEC61850_ErrorCodes
{
	IEC61850_ERROR_NONE								= 0,			/*!< Everything was ok */
	IEC61850_ERROR_INVALID_PARAMETERS 				= -1,		/*!< Supplied parameters are invalid */
	IEC61850_ERROR_NO_MEMORY 						= -2,		/*!< Allocation of memory has failed */
	IEC61850_ERROR_SCL_FILE_OPEN_FAILED 				= -3,		/*!< Provided SCL file failed to load */
	IEC61850_ERROR_SERVICE_FAILED		 				= -4,		/*!< Service failed to start */
	IEC61850_ERROR_SCL_SYNTAX_ERROR		 			= -5,		/*!< SCL File Failed to paser due to syntax error */
	IEC61850_ERROR_SCL_IO_ERROR		 				= -6,		/*!< SCL File Failed to paser due IO error */
	IEC61850_ERROR_SCL_NO_IED_CONNECTEDAP			= -7,		/*!< Can't find a matching ConnectedAP for this IED */
	IEC61850_ERROR_TYPE_MISMATCH						= -8,		/*!< The Type you are trying to write/update does not match the type in the server/client */
	IEC61850_ERROR_LICENCE_INVALID					= -9,		/*!< The licence file is not valid or present */
	IEC61850_ERROR_DATA_ATTRIBUTE_ID_NOT_FOUND		= -10,		/*!< The given Data Attribute ID was not found in the loaded SCL file */
	IEC61850_ERROR_DATA_ATTRIBUTE_ID_NO_CONNECTION = -11,		/*!< There is no ready connection to given Data Attribute ID */
	IEC61850_ERROR_SCL_LN_TYPE_NOT_FOUND			= -12,		/*!< The LN Type specifed in the LN element was not found in the data template */
	IEC61850_ERROR_SCL_DATASET_NOT_FOUND			= -13,		/*!< The Dataset specifed in a the control block was not found */
	IEC61850_ERROR_SCL_GSE_COMM_NOT_FOUND			= -14,		/*!< The GSE Communication access point specifed in a the control block was not found */
	IEC61850_ERROR_SCL_SV_COMM_NOT_FOUND			= -15,		/*!< The Sampled Value Communication access point specifed in a the control block was not found */
	IEC61850_ERROR_SCL_INVALID_MAC_ADDRESS			= -16,		/*!< The SCL contains a Invalid MAC address */
	IEC61850_ERROR_GOOSE_INIT_FAILED					= -17,		/*!< GOOSE Service failed to initialise */
	IEC61850_ERROR_SV_INIT_FAILED						= -18,		/*!< Sampled Value Service failed to initialise */
	IEC61850_ERROR_MMS_INIT_FAILED					= -19,		/*!< The MMS Service failed to initialise  */
	IEC61850_ERROR_SCL_DATA_TYPE_TEMPLATE_NAME		= -20,		/*!< Missing or invalid Data type name in Data Type Template  */
	IEC61850_ERROR_SCL_DATA_TYPE_TEMPLATE_ID		= -21,		/*!< Missing or invalid Data type ID in Data Type Template  */
	IEC61850_ERROR_SCL_DATASET_NAME					= -22,		/*!< Missing or invalid Data set Name  */
	IEC61850_ERROR_SCL_IED_ELEMENT_NAME				= -23,		/*!< Missing or Invalid IED element name (eg name attribute for IED, LD, LN, DOI, SDI or DAI) */
	IEC61850_ERROR_SCL_COMM_NAME					= -24, 		/*!< Missing or Invalid Communication element name  */
	IEC61850_ERROR_INVALID_STATE						= -25, 		/*!< The function cannot be performed while the client/server object is in the current state */
	IEC61850_ERROR_SCL_DATASET_FCDA_NOT_FOUND		= -26, 		/*!< The Functional Constrained Data Attribute in the dataset cannot be found */
	IEC61850_ERROR_SCL_DUPLICATE_LD					= -27, 		/*!< Two or more LDs have the same name. LDs names must be unique */
	IEC61850_ERROR_SCL_LD_NO_NAME 					= -28,		/*!< A Logical Device has no name. LD must have a name. */
	IEC61850_ERROR_SCL_LD_NO_LNS 						= -29,		/*!< The logical device has no Logical Nodes. LDs must have one Logical Node */
	IEC61850_ERROR_SCL_DUPLICATE_LN					= -30, 		/*!< Two or more LNs have the same name within a LD. LN names must be unique within a LD */
	IEC61850_ERROR_SCL_DUPLICATE_DOTYPES				= -31,		/*!< The DataType template contains duplicate DO Types. */
	IEC61850_ERROR_SCL_DUPLICATE_DATYPES				= -32,		/*!< The DataType template contains duplicate DA Types. */
	IEC61850_ERROR_SCL_DUPLICATE_ENUMTYPE			= -33, 		/*!< The DataType template contains duplicate Enumerated Types  */
	IEC61850_ERROR_SCL_DUPLICATE_LNTYPES				= -34, 		/*!< The DataType template contains duplicate Logical Node Types  */
	IEC61850_ERROR_SCL_DUPLICATE_LNTYPE_DO_NAME	= -35, 		/*!< The DataType template contains duplicatd Data Objects within a Logical Node Type */
	IEC61850_ERROR_SCL_LNTYPE_DO_NO_NAME			= -36,		/*!< The Data Object within a logical node type has no name */
	IEC61850_ERROR_SCL_LNTYPE_NO_DOS					= -37,		/*!< The Logical Node type within the data type template has no data objects */
	IEC61850_ERROR_SCL_DUPLICATE_DOTYPE_DA_NAME	= -38,		/*!< A Data Attribute within a Data Object type with the data type template has a name which is not unique */
	IEC61850_ERROR_SCL_DUPLICATE_DOTYPE_SDO_NAME 	= -39,		/*!< A Sub Data object within a Data Object type with the data type template has a name which is not unique */
	IEC61850_ERROR_SCL_DUPLICATE_DATYPE_BDA_NAME 	= -40,		/*!< A Basic Data Attribute within a Data attribute type with the data type template has a name which is not unique */
	IEC61850_ERROR_GOOSE_CONTROL_BLOCK_INVALID		= -41,		/*!< One of the GOOSE Control blocks is not valid and can't be started */
	IEC61850_ERROR_SCL_DUPLICATE_DA_ID				= -42,		/*!< There are two or more Data attributes that have the same DA ID (Private field) */
	IEC61850_ERROR_CONTROL_NOT_ACTIVE				= -43,		/*!< The given control is not active */
	IEC61850_ERROR_SCL_CONTROL_MODEL_INVALID		= -44,		/*!< The given control model found in the SCL file is not valid */
	IEC61850_ERROR_SCL_DAI_WITHOUT_DA_TYPE			= -45,		/*!< A Data Attribute (DAI) has no matching Data attribute type (DA) */
	IEC61850_ERROR_SCL_CLIENT_TOO_FEW_IEDS			= -46,		/*!< The SCD file does not contain 2 or more IEDs (Minimum of one IED for the client and one IED for a server) */
	IEC61850_ERROR_SERVICE_UNSUPPORTED				= -47,		/*!< A Service (eg GOOSE, MMS, Reports, SampleValue, Logging) is unsupported */
	IEC61850_ERROR_SV_CONTROL_BLOCK_INVALID			= -48,		/*!< One of the SV Control blocks is not valid and can't be started */
	IEC61850_ERROR_MMS_COMMS_FAILED					= -49,		/*!<  MMS layer Communication failed*/
	IEC61850_ERROR_MMS_SAP_FAILED					= -50};		/*!< Unable to create a service access point for the MMS server  */

/*! IEC 61850 Data Attribute IDs   */
enum IEC61850_DataAttributeIDs
{
	IEC61850_DAID_IEC101 		= 1,	/*!< IEC101 protocol */
	IEC61850_DAID_IEC103 		= 2,	/*!< IEC103 protocol */
	IEC61850_DAID_IEC104 		= 3,	/*!< IEC104 protocol */
	IEC61850_DAID_MODBUS		= 4,	/*!< ModBus protocol */
	IEC61850_DAID_DNP3			= 5,	/*!< DNP3 protocol */
	IEC61850_DAID_GENERIC		= 6,	/*!< Generic protocol */
	IEC61850_DAID_SYSTEMCORP	= 7,	/*!< SystemCorp Objects */
	IEC61850_DAID_DAP			= 8,	/*!< DAP Objects */
};

/*!	 */
typedef unsigned int daid_t;

/*!	 */
#define __DAID_COMMON(daid_prefix)  \
	daid_t daid_prefix##type	

/*!	\brief  Size of  daid_prefix */
#define __DAID_COMMON_SIZE  (sizeof (unsigned int))

/*!	\brief 		This structure hold the identification of a IEC61850 data Attribute. This is used to type cast from other Data Attributes ID Types. */
struct IEC61850_DataAttributeID
{
	__DAID_COMMON(daid_);			/*!< Common data attribute ID */
	char DAID_data[5*sizeof(unsigned int)]; 	/*!<  Common data */
};

/*!	\brief 		This structure hold the identification of a IEC61850 Data Attribute with a Generic ID. */
struct IEC61850_DataAttributeID_Generic
{
	__DAID_COMMON(Generic_);		/*!< This should be set to IEC61850_DAID_GENERIC  */
	unsigned int 	uiField1;		/*!< Generic Protocol Field value 1 */
	unsigned int 	uiField2;		/*!< Generic Protocol Field value 2 */
	unsigned int 	uiField3;		/*!< Generic Protocol Field value 3 */
	unsigned int 	uiField4;		/*!< Generic Protocol Field value 4 */
	unsigned int 	uiField5;		/*!< Generic Protocol Field value 5 */
};

/*!	\brief 		This structure hold the identification of a IEC61850 Data Attribute with a DNP3 ID. */
struct IEC61850_DataAttributeID_DNP3
{
	__DAID_COMMON(DNP3_); 	/*!< This should be set to IEC61850_DAID_DNP3  */
	unsigned int 	uiDatatype;			/*!< DataType for DNP3 */
	unsigned int 	uiObjectno;			/*!< Object Number for DNP3 */
	char zero[sizeof (struct IEC61850_DataAttributeID) - __DAID_COMMON_SIZE - (2*sizeof (unsigned int))]; /*!< Unused Data. This should be set to zero */
};

/*!	\brief 		This structure hold the identification of a IEC61850 Data Attribute with a IEC 101 object ID. */
struct IEC61850_DataAttributeID_IEC101
{
	__DAID_COMMON(IEC101_);	/*!< This should be set to IEC61850_DAID_IEC101  */
	unsigned int 	uiCASDU;			/*!< Common Address of SDU */
	unsigned int 	uiIOA;				/*!< Information Object Address */
	unsigned int 	uiTI;				/*!< Type Identification */
	unsigned char zero[sizeof (struct IEC61850_DataAttributeID) - __DAID_COMMON_SIZE - (3*sizeof (unsigned int))]; /*!< Unused Data. This should be set to zero */
};

/*!	\brief 		This structure hold the identification of a IEC61850 Data Attribute with a IEC 103 object ID. */
struct IEC61850_DataAttributeID_IEC103
{
	__DAID_COMMON(IEC103_);	/*!< This should be set to IEC61850_DAID_IEC103  */
	unsigned int 	uiRTUaddr;			/*!< RTU Address */
	unsigned int 	uiIOA;				/*!< Information Object Address */
	unsigned int 	uiTI;				/*!< Type Identification */
	unsigned char zero[sizeof (struct IEC61850_DataAttributeID) - __DAID_COMMON_SIZE - (3*sizeof (unsigned int))]; /*!< Unused Data. This should be set to zero */
};

/*!	\brief 		This structure hold the identification of a IEC61850 Data Attribute with a IEC 104 object ID. */
struct IEC61850_DataAttributeID_IEC104
{
	__DAID_COMMON(IEC104_);	/*!< This should be set to IEC61850_DAID_IEC104  */
	unsigned int 	uiCASDU;			/*!< Common Address of SDU */
	unsigned int 	uiIOA;				/*!< Information Object Address */
	unsigned int 	uiTI;				/*!< Type Identification */
	unsigned char zero[sizeof (struct IEC61850_DataAttributeID) - __DAID_COMMON_SIZE - (3*sizeof (unsigned int))]; /*!< Unused Data. This should be set to zero */
};

/*!	\brief 		This structure hold the identification of a IEC61850 Data Attribute with a DAP object ID. */
struct IEC61850_DataAttributeID_DAP
{
	__DAID_COMMON(DAP_);	/*!< This should be set to IEC61850_DAID_IEC104  */
	unsigned int 	uiID;			/*!< ID */
	unsigned int 	uiType;			/*!< Type */
	unsigned char zero[sizeof (struct IEC61850_DataAttributeID) - __DAID_COMMON_SIZE - (2*sizeof (unsigned int))]; /*!< Unused Data. This should be set to zero */
};

/*!	\brief 	struct IEC61850_DataAttributeID_ModBus.uiDataType Values 	*/
enum ModBusDataTypes
{
	MODBUS_DATATYPE_DISCRETE_INPUT			= 1,	 /*!< A Discrete input */
	MODBUS_DATATYPE_INPUT_REGISTER			= 2,	 /*!< A input Register */
	MODBUS_DATATYPE_COIL 						= 3,	 /*!< A Coil output */	
	MODBUS_DATATYPE_HOLDING_REGISTER		= 4, /*!< A Holding Register output */
};
/*!	\brief 		This structure hold the identification of a IEC61850 Data Attribute with a ModBus ID. */
struct IEC61850_DataAttributeID_ModBus
{
	__DAID_COMMON(ModBus_);	/*!< This should be set to IEC61850_DAID_MODBUS  */
	unsigned int 	uiSlaveAddress;		/*!< Slave Address in ModBus */
	unsigned int 	uiDataAddress;		/*!< DataAddress in ModBus */
	unsigned int 	uiDataType;			/*!< DataType in ModBus (see #ModBusDataTypes for values) */
	unsigned char zero[sizeof (struct IEC61850_DataAttributeID) - __DAID_COMMON_SIZE - (3*sizeof (unsigned int))]; /*!< Unused Data. This should be set to zero */
};

/*!	\brief 	struct IEC61850_DataAttributeID_SystemCORP.uiField Values 	*/
enum SystemCORP_Fields
{
	PIS10_FIELDS_DATA 						= 0,	 /*!< Data Field */
	PIS10_FIELDS_STATUS					= 1,	 /*!< Status Field */
	PIS10_FIELDS_TIMESTAMP				= 2,	 /*!< Time Stamp Field */
	PIS10_FIELDS_SELECT 					= 3,	 /*!< Select Field */	
	PIS10_FIELDS_OPERATE					= 4, /*!< Operate Field */
	PIS10_FIELDS_CANCEL					= 5, /*!< Cancel Field */
	PIS10_FIELDS_COMMAND_TERMINATION	= 6, /*!< Command Termination Field */
};
/*!	\brief 		This structure hold the identification of a IEC61850 Data Attribute with a SystemCORP ID. */
struct IEC61850_DataAttributeID_SystemCORP
{
	__DAID_COMMON(SystemCORP_);	/*!< This should be set to IEC61850_DAID_PIS10  */
	unsigned int 	uiCFENumber;			/*!< The ID Number of the CFE that this data belongs. (Equal to CFENo attribute in the PIS10 schema) */
	unsigned int 	uiClass;					/*!< The Data Class of this data instance (Equal to Class attribute in the PIS10 schema) */
	unsigned int 	uiNumber;				/*!< The Object number of this data instance  (Equal to Number attribute in the PIS10 schema)*/
	unsigned int 	uiField;					/*!< The data field this data belongs. (Data, Stauts, Timestamp, Select and Operate) see #SystemCORP_Fields (Equal to Field attribute in the PIS10 schema) */
	unsigned int 	uiCommandPort;			/*!< The Command Port of this object */
};

/*!	\brief 		A Data object structure. Used to exchange data objects between IEC61850 object and application. */
struct IEC61850_DataAttributeData
{
	unsigned char	ucType;			/*!< Type of data. Values can from #IEC61850_DataTypes */
	unsigned int 	uiBitLength;		/*!< Bit Length of data at pvData (NOTE: This is in Bits! So one octel equal to 8)*/
	void * 		pvData;			/*!< Pointer to data of length equal to uiBitLength bits */
};

/*!	IEC61850 Data Types (see IEC 61850-7-2:2003 section 5.5.2)  */
enum IEC61850_DataTypes
{
	IEC61850_DATATYPE_BOOLEAN			= 1,		/*!< Data is of type Boolean. If value is equal to 0 then false, otherwise it true  */
	IEC61850_DATATYPE_INT8				= 2,		/*!< A interger of 8 bits.  */
	IEC61850_DATATYPE_INT16				= 3,		/*!< A interger of 16 bits. */
	IEC61850_DATATYPE_INT32				= 4,		/*!< A interger of 32 bits.  */
	IEC61850_DATATYPE_INT64				= 5,		/*!< A integer of 64 bits */
	IEC61850_DATATYPE_INT8U				= 6,		/*!< A unsigned interger of 8 bits. */
	IEC61850_DATATYPE_INT16U			= 7,		/*!< A unsigned interger of 16 bits. */
	IEC61850_DATATYPE_INT32U			= 8,		/*!< A unsigned interger of 32 bits. */
	IEC61850_DATATYPE_FLOAT32			= 9,		/*!< A IEEE 754 single precision floating point */
	IEC61850_DATATYPE_FLOAT64			= 10,		/*!< A IEEE 754 double precision floating point */
	IEC61850_DATATYPE_ENUMERATED		= 11,		/*!< Ordered set of values. exetended allowed */
	IEC61850_DATATYPE_CODED_ENUM		= 12,		/*!< Ordered set of values. Not allowed to be exetended */
	IEC61850_DATATYPE_OCTEL_STRING		= 13,		/*!< A String of Octels charactures  */
	IEC61850_DATATYPE_VISIBLE_STRING	= 14,		/*!< A String of Visible charactures  */
	IEC61850_DATATYPE_UNICODE_STRING	= 15,		/*!< A String of Unicode charactures  */
	IEC61850_DATATYPE_TIMESTAMP			= 16,		/*!< TimeStamp type (5.5.3.7.1 of IEC 61850-7-2:2003) */
	IEC61850_DATATYPE_QUALITY			= 17,		/*!< Quality data type. This a big endian bitstring as defined in enum IEC61850QualityFlags */
};


/*!	\brief  	Number of bits used in Boolean type 	 */
#define IEC61850_BOOLEAN_BITSIZE	8

/*!	\brief  	Size of struct IEC61850TimeStamp 	 */
#define IEC61850_TIMESTAMP_BITSIZE	64

/*!	\brief 	IEC61850 NTP Time Stamp Structure (as specified in IEC61850_DATATYPE_TIMESTAMP)	 */
struct IEC61850_TimeStamp
{	
	Unsigned32		u32Seconds;				/*!< Number of Seconds */
	Unsigned32		u32FractionsOfSecond;	/*!< Fraction of a second as a binary fraction. */
};

/*!  IEC61850 Quality flags. (see IEC 61850-8-1:2004 section 8.2)  */
enum IEC61850TimeQualityFlags
{				
	IEC61850_TIMEQUALITY_LEAP_SECOND_KNOWN				= 0x0000000080,	/*!<  */
	IEC61850_TIMEQUALITY_CLOCK_FAILURE					= 0x0000000040,	/*!<  */
	IEC61850_TIMEQUALITY_CLOCK_NOT_SYNCHRONIZED		= 0x0000000020,	/*!<  */
	IEC61850_TIMEQUALITY_0_BIT_OF_ACCURACY				= 0x0000000000,	/*!<  */
	IEC61850_TIMEQUALITY_1_BIT_OF_ACCURACY				= 0x0000000001,	/*!<  */
	IEC61850_TIMEQUALITY_24_BIT_OF_ACCURACY				= 0x0000000018,	/*!<  */
	IEC61850_TIMEQUALITY_ACCURACY_UNSPECIFIED			= 0x000000001F,	/*!<  */
};

/*!	\brief  	Number of bits used in Quality flag 	 */
#define IEC61850_QUALITY_BITSIZE	13

/*!	\brief 	IEC61850 Quality data type. (as specified in #IEC61850QualityFlags) */
typedef Unsigned16 tIEC61850Quality;

/*!	IEC61850 Quality flags. (see IEC 61850-8-1:2004 section 8.2) */
enum IEC61850QualityFlags
{		
	IEC61850_QUALITY_GOOD					= 0x0000,	/*!< Good */
	IEC61850_QUALITY_INVALID				= 0x4000,	/*!< Invalid */
	IEC61850_QUALITY_RESERVED				= 0x8000,	/*!< Reserved */
	IEC61850_QUALITY_QUESTIONABLE		= 0xC000,	/*!< Questionable */
	IEC61850_QUALITY_OVERFLOW			= 0x2000,	/*!< Overflow */
	IEC61850_QUALITY_OUTOFRANGE 			= 0x1000,	/*!< Out of Range */
	IEC61850_QUALITY_BADREFERENCE		= 0x0800,	/*!< Bad Reference */
	IEC61850_QUALITY_OSCILLATORY			= 0x0400,	/*!< Oscillatory */
	IEC61850_QUALITY_FAILURE				= 0x0200,	/*!< Failure */
	IEC61850_QUALITY_OLDDATA				= 0x0100,	/*!< Old Data */
	IEC61850_QUALITY_INCONSISTENT			= 0x0080,	/*!< Inconsistent */
	IEC61850_QUALITY_INACCURATE			= 0x0040,	/*!< Inaccurate */
	IEC61850_QUALITY_SUBSTITUED			= 0x0020,	/*!< Substitued if not set then Source is from the process */
	IEC61850_QUALITY_TEST					= 0x0010,	/*!< Test */
	IEC61850_QUALITY_OPERATOR_BLOCKED	= 0x0008,	/*!< Blocked by Operator */
};

/*!	\brief  	Bit mask of Validity flags (IEC61850_QUALITY_GOOD, IEC61850_QUALITY_INVALID,  IEC61850_QUALITY_RESERVED and IEC61850_QUALITY_QUESTIONABLE) 	 */
#define IEC61850_QUALITY_BITMASK_VALIDITY 0xC000

/*!	\brief  	Number of bits used in  Enum 	 */
#define IEC61850_ENUM_BITSIZE	8

/*!	\brief 	IEC61850 Enumerated data type. (as specified in IEC61850_DATATYPE_ENUMERATED or IEC61850_DATATYPE_CODED_ENUM) */
typedef Unsigned8 tIEC61850Enum;

/*!	\brief  Datatype of 	 Double position type	 */
#define IEC61850_DATATYPE_DBPOS  IEC61850_DATATYPE_INT8U

/*!	\brief  	Number of bits used in a Double position type (IEC61850_DATATYPE_DBPOS) 	 */
#define IEC61850_DBPOS_BITSIZE	8

/*!	enumerated value of double position as defined in IEC 61850 */
enum DbPosValues
{
	DBPOS_INTERMEDIATE		= 0,	/*!< intermediate  */
	DBPOS_OFF					= 1,	/*!< off */
	DBPOS_ON					= 2,	/*!< on */
	DBPOS_BAD					= 3,	/*!< bad */
};


/*!	Values of double point (e.g. for Dbpos.stVal) to be provided by update callback */
enum IEC61850DbPosValues
{
	IEC61850_DB_POS_OFF		= 0x00,	/*!< Double Position is intermediate state  */
	IEC61850_DB_POS_FALSE		= 0x40,	/*!< Double Position is false/open */		
	IEC61850_DB_POS_TRUE		= 0x80,	/*!< Double Position is close/true */
	IEC61850_DB_POS_INVALID	= 0xC0,	/*!< Double Position is in a bad or invalid state */
};

/*!    The states of Mode and Behavior (see 61850-7-4 IEC:2003 page 80)*/
enum IEC61850_ModeAndBehaviourStates
{
	IEC61850_MODE_ON 				= 1, /*!<  Normal state. */
	IEC61850_MODE_BLOCKED 		= 2, /*!<  Process is passively supervised. */
	IEC61850_MODE_TEST 			= 3, /*!<  Function is operated but results are indicated as test results. */
	IEC61850_MODE_TESTBLOCKED 	= 4, /*!<  Function is operated in test mode but with no impact to the proces.  */
	IEC61850_MODE_OFF 				= 5, /*!<  Function is inactive but shows its configuration capability. */ 
};

/*!    The states of Health (see 61850-7-4 IEC:2003 page 75) */
enum IEC61850_HealthStates
{
	IEC61850_HEALTH_OK			= 1,		/*!< OK ("Green") No problems, normal operation   */
	IEC61850_HEALTH_WARNING		= 2,		/*!< Warning (yellow ? minor problems, but in safe operation mode */
	IEC61850_HEALTH_ALARM			= 3,		/*!< Alarm (red? severe problem, no operation possible */
};

struct IEC61850_Struct; // Forward Declaration of struct

/*!	\brief 	Pointer to a IEC 61850 object */
typedef struct IEC61850_Struct * IEC61850;

/*!	\brief 	Read Callback definition. This function should return a value from the available IEC61850_CallbackReturnServiceErrorCodes */
typedef int (*IEC61850_ReadCallback)(void * ptUserData, struct IEC61850_DataAttributeID * ptDataAttributeID, struct IEC61850_DataAttributeData * ptReturnedValue);

/*!	\brief 	Write callback definition. This function should return a value from the available IEC61850_CallbackReturnServiceErrorCodes */
typedef int (*IEC61850_WriteCallback)(void * ptUserData, struct IEC61850_DataAttributeID * ptDataAttributeID, const struct IEC61850_DataAttributeData * ptNewValue);

/*!	\brief 	Update callback definition. */
typedef void (*IEC61850_UpdateCallback)(void * ptUserData, struct IEC61850_DataAttributeID * ptDataAttributeID, const struct IEC61850_DataAttributeData * ptNewValue);

/*!	\brief 	Command Select CallBack */
typedef int (*IEC61850_ControlSelectCallback)(void * ptUserData, struct IEC61850_DataAttributeID * ptControlID, const struct IEC61850_DataAttributeData * ptSelectValue, int iSyncroCheck, int iInterlockCheck);

/*!	\brief 	Command Operate CallBack */
typedef int (*IEC61850_ControlOperateCallback)(void * ptUserData, struct IEC61850_DataAttributeID * ptControlID, const struct IEC61850_DataAttributeData * ptOperateValue, int iSyncroCheck, int iInterlockCheck);

/*!	\brief 	Command Cancel CallBack */
typedef int (*IEC61850_ControlCancelCallback)(void * ptUserData, struct IEC61850_DataAttributeID * ptControlID);



/*!	Read Write and Update callbacks returned service error codes. (see IEC 61850-7-2:2003 section 5.5.3.4)  */
enum IEC61850_CallbackReturnServiceErrorCodes
{
	IEC61850_CB_ERROR_NONE									= 0,			/*!< Everything when OK */
	IEC61850_CB_ERROR_INSTANCE_NOT_AVAILABLE				= -1,		/*!< Action failed due to instance being not available */
	IEC61850_CB_ERROR_ACCESS_VIOLATION						= -2,		/*!< Action failed due to access violation */
	IEC61850_CB_ERROR_PARAMETER_VALUE_INCONSISTENT			= -3,		/*!< Action failed due to inconsistent parameter value */
	IEC61850_CB_ERROR_INSTANCE_LOCKED_BY_OTHER_CLIENT		= -4,		/*!< Action failed due to data locked by other client */
	IEC61850_CB_ERROR_TYPE_CONFLICT							= -5,		/*!< Action failed data type conflict */
	IEC61850_CB_ERROR_FAILED_DUE_TO_SERVER_CONSTRAINT		= -6,		/*!< Action failed due to server constraint */
};

/*!	Enumerated data type for command (controls) error codes */
enum eCommandAddCause
{
	IEC61850_COMMAND_ERROR_NONE 											= -1,		/*!< Control action successfully executed.  */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_UNKNOWN								= 0,			/*!< Default value. Command not successful due to Unknown causes */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_NOT_SUPPORTED						= 1,			/*!< Control action is not supported. */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_BLOCKED_BY_SWITCHING_HIERARCHY	= 2,			/*!< Not successful since one of the downstream Loc switches like in CSWI has the value TRUE */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_SELECT_FAILED						= 3,			/*!< Canceled due to an unsuccessful selection (select service) */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_INVALID_POSITION						= 4,			/*!< Control action is aborted due to invalid switch position (Pos in XCBR or XSWI) */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_POSITION_REACHED					= 5,			/*!< Switch is already in the intended position (Pos in XCBR or XSWI) */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_PARAMETER_CHANGE_IN_EXECUTION		= 6,			/*!< Control action is blocked due to running parameter change. */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_STEP_LIMIT							= 7,			/*!< Control action is blocked, because tap changer has reached the limit (EndPosR or EndPosL in YLTC). */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_BLOCKED_BY_MODE					= 8,			/*!< Control action is blocked, because the LN (CSWI or XCBR/XSWI) is in a mode (Mod) which doesnt allow any switching. */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_BLOCKED_BY_PROCESS					= 9,			/*!< Control action is blocked due to some external event at process level that prevents a successful operation, for example blocking indication (EEHealth in XCBR or XSWI). */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_BLOCKED_BY_INTERLOCKING			= 10,		/*!< Control action is blocked due to interlocking of switching devices (in CILO attribute EnaOpn.stVal=FALSE?or EnaCls.stVal=FALSE?. */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_BLOCKED_BY_SYNCHROCHECK			= 11,		/*!< Control action with synchrocheck is aborted due to exceed of time limit and missing synchronism condition. */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_COMMAND_ALREADY_IN_EXECUTION		= 12,		/*!< Control, select or cancel service is rejected, because control action is already running.  */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_BLOCKED_BY_HEALTH					= 13,		/*!< Control action is blocked due to some internal event that prevents a successful operation (Health). */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_1_OF_N_CONTROL						= 14,		/*!< Control action is blocked, because another control action in a domain (for example substation) is already running (in any XCBR or XSWI of that domain, the DPC.stSeld=TRUE?. */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_ABORTION_BY_CANCEL					= 15,		/*!< Control action is aborted due to cancel service. */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_TIME_LIMIT_OVER						= 16,		/*!< Control action is terminated due to exceed of some time limit. */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_ABORTION_BY_TRIP					= 17,		/*!< Control action is aborted due to a trip (PTRC with ACT.general=TRUE?. */
	IEC61850_COMMAND_ERROR_ADD_CAUSE_OBJECT_NOT_SELECTED					= 18,		/*!< Control action is rejected, because control object was not selected */
};

/*!    Server/client parameter  */
enum IEC61850_ClientServerFlag
{
	IEC61850_SERVER									= 0,		/*!< This IEC61850 Obbject is a Client */
	IEC61850_CLIENT										= 1,		/*!< This IEC61850 Obbject is a Server */
};

/*!	Flags for struct IEC61850_Parameters.uiOptions flags */
enum IEC61850_OptionFlag
{
	IEC61850_OPTION_NONE								= 0x00,		/*!< No options set */
	IEC61850_OPTION_FLAG_TEST_MODE					= 0x01,		/*!< Set the server in to test mode */
	IEC61850_OPTION_BLOCK_WRITE_TO_CF				= 0x02,		/*!< Allow Write Access to Configuration fields */
	IEC61850_OPTION_SERVER_SEND_CONCLUDE_REQUEST	= 0x04,		/*!< Enables Server send conclude request to Client */
	IEC61850_OPTION_SERVER_SUBSCRIBE_GOOSE			= 0x08,		/*!< Enable the server to receive GOOSE messages*/
	IEC61850_OPTION_SYSTEMCORP_RESERVED				= 0x80,		/*!< SystemCORP Reserved please do not  use */
};

/*!	\brief 		Create Server/client parameters structure  */
struct IEC61850_Parameters
{
	enum IEC61850_ClientServerFlag 		ClientServerFlag;		/*!< Flag set to indicate if this is to be a server  or a client  see #IEC61850_ClientServerFlag for values */
#ifndef __PARADIGM__	
	unsigned int 						uiOptions;			/*!< Options flag, used to set client/server global options see #IEC61850_OptionFlag for values */
	unsigned int						uiCmdTermTimeOut;	/*!< Command Termination time out in milliseconds . If 0 than defaults to 10000 milliseconds */
	unsigned int						uiMaxAssociations; 	/*!< Specify Max number of Associations, If 0  or greater than 16 then the default  max number of associations will be set to 16 */
#else
	unsigned long int 					uiOptions;			/*!< Options flag, used to set client/server global options see #IEC61850_OptionFlag for values */
	unsigned long int					uiCmdTermTimeOut;	/*!< Command Termination time out in milliseconds . If 0 than defaults to 10000 milliseconds */
	unsigned long int					uiMaxAssociations; 	/*!< Specify Max number of Associations, If 0  or greater than 16 then the default  max number of associations will be set to 16 */
#endif	
	unsigned int						uiClientMaxConnections; /*!< The maximum number of connections the client can handle. If 0 than defaults to  FD_SETSIZE */
	IEC61850_ReadCallback				ptReadCallback;		/*!< Read callback function. If equal to NULL then callback is not used. */
	IEC61850_WriteCallback				ptWriteCallback;		/*!< Write callback function. If equal to NULL then callback is not used. */
	IEC61850_UpdateCallback			ptUpdateCallback;		/*!< Update callback function. If equal to NULL then callback is not used. */

	IEC61850_ControlSelectCallback		ptSelectCallback;		/*!< Function called when a Select (SBO) or Select with value (SBOw) command is executed */
	IEC61850_ControlOperateCallback		ptOperateCallback;	/*!< Function called when a Operate command is executed */
	IEC61850_ControlCancelCallback		ptCancelCallback;		/*!< Function called when a Cancel command is executed */
};

struct IEC61850_ControlParameters
{
	int iSyncroCheck;
	int iInterlockCheck;
	int iTimeout;
};



#ifdef __cplusplus
}
#endif

#endif
// End of File
