/*****************************************************************************/
/*!	\file 			QtClientWidget.cpp
 *	\brief                  Qt Client Widget
 *	\par 			SystemCORP Embedded Technology Pty Ltd
 *
 *				15/50 William St.,
 *				Beckenham,
 *				Perth, WA 6107,
 *				Australia.
 *
 *				Phone	: +61 (8) 9258 4683
 *				Fax	: +61 (8) 9258 4673
 *				Email	: info@systemcorp.com.au
 */
/*****************************************************************************/

/******************************************************************************
*	Include
******************************************************************************/
#include <QtGui>
#include <QtGui/QApplication>
#include <QGraphicsView>
#include <QGraphicsPixmapItem>
#include <QPixmap>
#include <QLabel>
#include <qpixmap.h>
#include <QTimeLine>
#include <time.h>
#include "QtClientWidget.h"
#include "ui_QtClientWidget.h"
#include "pis10Client.h"

/******************************************************************************
* Status Variables
******************************************************************************/
#define MaxCount 999
#define IEC61850_CLIENT_INDEX 0

unsigned int phsABIn;
unsigned int phsAIn;
int imaxAlarmThreshold;

Boolean iAlarmFlg;
static char iToggle=0;

// DipSW Status flags
Boolean bDipSw1 =FALSE;
Boolean bDipSw2 =FALSE;
Boolean bDipSw3 =FALSE;
Boolean bDipSw4 =FALSE;
// Led Status flags
Boolean bLed1 =FALSE;
Boolean bLed2 =FALSE;
Boolean bLed3 =FALSE;
Boolean bLed4 =FALSE;
// XCBR CSWI Status flags
Boolean bErthSwFlg = 0;
Boolean bXCBRSwFlg = 0;
Boolean bUpdtErthSwFlg = 0;
Boolean bUpdtXCBRSwFlg = 0;

/*****************************************************************************
* Constructor
******************************************************************************
* Constructor for QtClientWidget.
*
******************************************************************************/
QtClientWidget::QtClientWidget(QWidget *parent) : QWidget(parent) //ui(new Ui::QtServerWidget)
{
        setupUi(this);
        setWindowFlags(Qt::FramelessWindowHint);
        QImage imgLogo("TQ Demo Banner.png");

        scene = new QGraphicsScene;

        // Load the LED image files
        pixmapGreenLed = new QPixmap("square_green.svg");
        pixmapRedLed = new QPixmap("square_red.svg");
        pixmapGreyLed = new QPixmap("square_grey.svg");
        pixmapOrangeLed = new QPixmap("square_orange.svg");

        IconGreenLed = new QIcon("square_green.svg");
        IconRedLed= new QIcon("square_red.svg");

        QPixmap pixmapESWOpenXCBROpen("switch00.png");
        QPixmap pixmapESWOpenXCBRClose("switch01.png");
        QPixmap pixmapESWCloseXCBROpen("switch10.png");
        QPixmap pixmapESWCloseXCBRClose("switch11.png");

        scene->addPixmap(pixmapESWOpenXCBROpen);

        //ui->
        gvSw->setScene(scene);
        //ui->
        gvSw->show();
        //ui->
        lblLogo->setPixmap(QPixmap::fromImage(imgLogo));

        // Creating Timer for constant updates
        timerUpdateState = new QTimer(this);
        timerAlarmColor = new QTimer(this);

        // Set Default values
        bErthSwFlg = 0;
        bXCBRSwFlg = 0;
        imaxAlarmThreshold = 0;
        //scene->addPixmap(pixmapESWCloseXCBROpen);

        connect(timerAlarmColor, SIGNAL(timeout()), this, SLOT(AlarmBtnColorChange()));

        //Timers to handle the flashing of the buttons after an Operate has been performed.
        connect(timerUpdateState, SIGNAL(timeout()), this, SLOT(UpdateStateTimerHandler()));
        timerUpdateState->start(500);
}

/*****************************************************************************
* Destructor
******************************************************************************
* Destructor for for QtServerWidget.
*
******************************************************************************/
QtClientWidget::~QtClientWidget()
{
    delete this;
}

// Handler functions
/*****************************************************************************
* pbDipSw1Handler() Function
******************************************************************************
* This function is handler for DipSw1 it gets called when DipSw1 is clicked
*
******************************************************************************/
void QtClientWidget ::pbDipSw1Handler()
{
    char cValue = 0;
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;
    struct IEC61850_ControlParameters tControlParams;

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(cValue)*(8);
    tDAData.pvData = &cValue;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 2;
    tDAID.uiField2 = 1;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    memset(&tControlParams, 0, sizeof(tControlParams));

    if(bDipSw1 == TRUE)
    {
        bDipSw1 = FALSE;
        pbDipSw1->setIcon(*IconRedLed);
    }
    else
    {
        bDipSw1 = TRUE;
        pbDipSw1->setIcon(*IconGreenLed);
    }

    cValue = bDipSw1;

    //printf("Operating Led1  and DIP Sw to False\n");
    eError = IEC61850_ControlOperate(PIS10Client::pThis->GetMyClient(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, tControlParams);
    if(eError == IEC61850_ERROR_NONE)
    {
        printf("Operate value is %d \n", bDipSw1);
    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
    }
}

/*****************************************************************************
* pbDipSw1Handler() Function
******************************************************************************
* This function is handler for DipSw1 it gets called when DipSw1 is clicked
*
******************************************************************************/
void QtClientWidget ::pbDipSw2Handler()
{
    char cValue = 0;
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;
    struct IEC61850_ControlParameters tControlParams;

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(cValue)*(8);
    tDAData.pvData = &cValue;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 2;
    tDAID.uiField2 = 2;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    memset(&tControlParams, 0, sizeof(tControlParams));

    if(bDipSw2 == TRUE)
    {
        bDipSw2 = FALSE;
        pbDipSw2->setIcon(*IconRedLed);
    }
    else
    {
        bDipSw2 = TRUE;
        pbDipSw2->setIcon(*IconGreenLed);
    }

    cValue = bDipSw2;
    //printf("Operating Led1  and DIP Sw to False\n");
    eError = IEC61850_ControlOperate(PIS10Client::pThis->GetMyClient(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, tControlParams);

   if(eError == IEC61850_ERROR_NONE)
    {
        printf("Write value is %d \n", bDipSw2);
    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
    }
}

/*****************************************************************************
* pbDipSw1Handler() Function
******************************************************************************
* This function is handler for DipSw1 it gets called when DipSw1 is clicked
*
******************************************************************************/
void QtClientWidget ::pbDipSw3Handler()
{
    char cValue = 0;
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;
    struct IEC61850_ControlParameters tControlParams;

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(cValue)*(8);
    tDAData.pvData = &cValue;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 2;
    tDAID.uiField2 = 3;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    memset(&tControlParams, 0, sizeof(tControlParams));

    if(bDipSw3 == TRUE)
    {
        bDipSw3 = FALSE;
        //Led_Lbl3->setPixmap(*pixmapRedLed);
        pbDipSw3->setIcon(*IconRedLed);
    }
    else
    {
        bDipSw3 = TRUE;
        //Led_Lbl3->setPixmap(*pixmapGreenLed);
        pbDipSw3->setIcon(*IconGreenLed);
    }

    cValue = bDipSw3;
    //printf("Operating Led1  and DIP Sw to False\n");
    eError = IEC61850_ControlOperate(PIS10Client::pThis->GetMyClient(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, tControlParams);

    if(eError == IEC61850_ERROR_NONE)
    {
        printf("Write value is %d \n", bDipSw3);
    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
    }
}

/*****************************************************************************
* pbDipSw1Handler() Function
******************************************************************************
* This function is handler for DipSw1 it gets called when DipSw1 is clicked
*
******************************************************************************/
void QtClientWidget ::pbDipSw4Handler()
{
    char cValue = 0;
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;
    struct IEC61850_ControlParameters tControlParams;

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(cValue)*(8);
    tDAData.pvData = &cValue;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 2;
    tDAID.uiField2 = 4;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    memset(&tControlParams, 0, sizeof(tControlParams));

    if(bDipSw4 == TRUE)
    {
        bDipSw4 = FALSE;
        //Led_Lbl4->setPixmap(*pixmapRedLed);
        pbDipSw4->setIcon(*IconRedLed);
    }
    else
    {
        bDipSw4 = TRUE;
        //Led_Lbl4->setPixmap(*pixmapGreenLed);
        pbDipSw4->setIcon(*IconGreenLed);
    }

    cValue = bDipSw4;
    //printf("Operating Led1  and DIP Sw to False\n");
    eError = IEC61850_ControlOperate(PIS10Client::pThis->GetMyClient(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, tControlParams);

    if(eError == IEC61850_ERROR_NONE)
    {
        printf("Write value is %d \n", bDipSw4);
    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
    }
}

/*****************************************************************************
* pshbtnWriteAnIn1Handler() Function
******************************************************************************
* This function is handler for AnIn1 it gets called when AnIn1 is clicked to
* set a value
******************************************************************************/
void QtClientWidget ::pshbtnReadAnIn1Handler()
{
    int iAnIn1;
    char cAnIn1[5] = {0};

    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;

    tDAData.ucType = IEC61850_DATATYPE_INT32U;
    tDAData.uiBitLength = sizeof(iAnIn1)*8;
    tDAData.pvData = &iAnIn1;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 3;
    tDAID.uiField2 = 1;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    //Read the value from server
    eError = IEC61850_Read(PIS10Client::pThis->GetMyClient(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData);
    if(eError == IEC61850_ERROR_NONE)
    {
        printf("Read value is %d \n", iAnIn1);
        sprintf(cAnIn1,"%d",iAnIn1 );
        textEditAnIn1->setText(cAnIn1);
    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
    }
}

/*****************************************************************************
* pshbtnWriteAnIn2Handler() Function
******************************************************************************
* This function is handler for AnIn2 it gets called when AnIn2 is clicked to
* set a value
******************************************************************************/
void QtClientWidget ::pshbtnReadAnIn2Handler()
{
    unsigned int iAnIn2;
    char cAnIn2[5] = {0};
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;

    tDAData.ucType = IEC61850_DATATYPE_INT32U;
    tDAData.uiBitLength = sizeof(iAnIn2)*8;
    tDAData.pvData = &iAnIn2;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 3;
    tDAID.uiField2 = 2;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    //Read the value from server
    eError = IEC61850_Read(PIS10Client::pThis->GetMyClient(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData);
    if(eError == IEC61850_ERROR_NONE)
    {
        printf("Update value is %d \n", iAnIn2);
        //lcdNumAnIn2->display((float)iAnIn2);
        sprintf(cAnIn2,"%d",iAnIn2 );
        textEditAnIn2->setText(cAnIn2);
    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);

    }
}

/*****************************************************************************
* pbAlrmHandler() Function
******************************************************************************
* This function is handler for AlramThreshold it gets called when AlramThreshold
* is clicked to set a value
******************************************************************************/
void QtClientWidget :: pbAlrmHandler()
{
    unsigned int iAlrm = 0;
    char cAnlrm[5] = {0};
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;

    tDAData.ucType = IEC61850_DATATYPE_INT32U;
    tDAData.uiBitLength = sizeof(iAlrm)*8;
    tDAData.pvData = &iAlrm;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 1;
    tDAID.uiField2 = 6;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    eError = IEC61850_Read(PIS10Client::pThis->GetMyClient(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData);
    if(eError == IEC61850_ERROR_NONE)
    {
        printf("Update value is %d \n", iAlrm);
        sprintf(cAnlrm,"%d",iAlrm );
        textEditAlrm->setText(cAnlrm);
    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
    }
}

/*****************************************************************************
* LedController() Function
******************************************************************************
* This function is handler for Leds when client performs operate on LEDs
* IEC61850_OperateCallback function calls this function to set the flags
******************************************************************************/
void QtClientWidget ::LedController(unsigned int uiLedIndx, Boolean bValue)
{
    Boolean bLedStatus = FALSE;

    //printf("LedController=> uiLedIndx: %d, bValue: %d\n",uiLedIndx,bValue );

    switch(uiLedIndx)
    {
        case 1:
        {
            if(bValue == TRUE)
            {
                bLed1 = TRUE;
                bLedStatus = TRUE;
            }
            else
            {
                bLed1 = FALSE;
                bLedStatus = FALSE;
            }
        }
        break;
        case 2:
        {
            if(bValue == TRUE)
            {
                bLed2 = TRUE;
                bLedStatus = TRUE;
            }
            else
            {
                bLed2 = FALSE;
                bLedStatus = FALSE;
            }
        }
        break;
        case 3:
        {
            if(bValue == TRUE)
            {
                bLed3 = TRUE;
                bLedStatus = TRUE;
            }
            else
            {
                bLed3 = FALSE;
                bLedStatus = FALSE;
            }
        }
        break;
        case 4:
        {
            if(bValue == TRUE)
            {
                bLed4 = TRUE;
                bLedStatus = TRUE;
            }
            else
            {
                bLed4 = FALSE;
                bLedStatus = FALSE;
            }
        }
        break;
        case 5:
        {
            if(bValue == TRUE)
            {
                iAlarmFlg = TRUE;
            }
            else
            {
                iAlarmFlg = FALSE;
            }
        }
        break;
        }
}

/*****************************************************************************
* XCBR_CSWIController() Function
******************************************************************************
* This function is handler for XCBR & CSW Sw, when client performs operate on XCBR & CSW
* IEC61850_OperateCallback function calls this function to set the flags
******************************************************************************/
void QtClientWidget ::XCBR_CSWIController(unsigned int uiIndx, unsigned char cValue)
{
    //printf("XCBR_CSWIController=> uiIndx: %d, cValue: %d\n",uiIndx,cValue );

    switch(uiIndx)
    {
        case 4:
        {
            if(cValue == IEC61850_DB_POS_TRUE)
            {
                bUpdtXCBRSwFlg = TRUE;
            }
            else if(cValue == IEC61850_DB_POS_FALSE)
            {
                bUpdtXCBRSwFlg = FALSE;
            }
        }
        break;
        case 5:
        {
            if(cValue == IEC61850_DB_POS_TRUE)
            {
                bUpdtErthSwFlg = TRUE;
            }
            else if(cValue == IEC61850_DB_POS_FALSE)
            {
                bUpdtErthSwFlg = FALSE;
            }
        }
        break;
        }
}

/*****************************************************************************
* pbXcbrHandler() Function
******************************************************************************
* This function is handler for XCBR & CSW Sw, when client performs operate on XCBR & CSW
* IEC61850_OperateCallback function calls this function to set the flags
******************************************************************************/
void QtClientWidget ::pbXcbrHandler()
{
    char cValue = 0;
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;
    struct IEC61850_ControlParameters tControlParams;

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(cValue)*(8);
    tDAData.pvData = &cValue;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 4;
    tDAID.uiField2 = 1;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    memset(&tControlParams, 0, sizeof(tControlParams));

    if(bErthSwFlg) // Check if CSWI is switched on
    {
        //Pop a warning message
        QRect screenRect = QWidget().rect();
        QMessageBox msg;
        msg.setText(tr("CSWI is switched ON \n Switch CSWI OFF "));
        QSize mSize = msg.sizeHint();
        msg.setFixedSize(80,40);
        msg.setIcon(QMessageBox::Warning);
        msg.move(QPoint( screenRect.width()/2 - mSize.width()/2,screenRect.height()/2 - mSize.height()/2 ));
        setFocus(Qt::OtherFocusReason);
        msg.exec();
    }
    else
    {
        if(bXCBRSwFlg == TRUE)
        {
            bXCBRSwFlg = FALSE;
            pbXCBR->setIcon(*IconRedLed);
        }
        else
        {
            bXCBRSwFlg = TRUE;
            pbXCBR->setIcon(*IconGreenLed);
        }

        cValue = bXCBRSwFlg;
        eError = IEC61850_ControlOperate(PIS10Client::pThis->GetMyClient(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, tControlParams);
        if(eError == IEC61850_ERROR_NONE)
        {
            printf("Operate XCBR value is %d \n", bXCBRSwFlg);
        }
        else
        {
            printf("Got error on write %d \n", eError);
            printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
        }
    }
}

/*****************************************************************************
* pbCswiHandler() Function
******************************************************************************
* This function is handler for XCBR & CSW Sw, when client performs operate on XCBR & CSW
* IEC61850_OperateCallback function calls this function to set the flags
******************************************************************************/
void QtClientWidget ::pbCswiHandler()
{
    char cValue = 0;
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;
    struct IEC61850_ControlParameters tControlParams;

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(cValue)*(8);
    tDAData.pvData = &cValue;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 5;
    tDAID.uiField2 = 1;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    memset(&tControlParams, 0, sizeof(tControlParams));

    if(bXCBRSwFlg) // Check if XCBR is switched on
    {
        //Pop a warning message
        QRect screenRect = QWidget().rect();
        QMessageBox msg;
        msg.setText(tr("XCBR is switched ON \n Switch XCBR OFF "));
        QSize mSize = msg.sizeHint();

        msg.setFixedSize(80,40);
        msg.setIcon(QMessageBox::Warning);
        msg.move(QPoint( screenRect.width()/2 - mSize.width()/2,screenRect.height()/2 - mSize.height()/2 ));
        setFocus(Qt::OtherFocusReason);
        msg.exec();
    }
    else
    {
        if(bErthSwFlg == TRUE)
        {
            bErthSwFlg = FALSE;
            pbCSWI->setIcon(*IconRedLed);
        }
        else
        {
            bErthSwFlg = TRUE;
            pbCSWI->setIcon(*IconGreenLed);
        }

        cValue = bErthSwFlg;

        eError = IEC61850_ControlOperate(PIS10Client::pThis->GetMyClient(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, tControlParams);
        if(eError == IEC61850_ERROR_NONE)
        {
            printf("Operate CSWI value is %d \n", bErthSwFlg);
        }
        else
        {
            printf("Got error on write %d \n", eError);
            printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
        }
    }
}

/*****************************************************************************
* UpdateStateTimerHandler() Function
******************************************************************************
* Timer function handler, is called when timer fires. This functions checks the
* Global status flags and updates  QServerwidget GUI
******************************************************************************/
void QtClientWidget::UpdateStateTimerHandler()
{
    QPixmap pixmapESWOpenXCBROpen("switch00.png");
    QPixmap pixmapESWOpenXCBRClose("switch01.png");
    QPixmap pixmapESWCloseXCBROpen("switch10.png");
    QPixmap pixmapESWCloseXCBRClose("switch11.png");

    if(bLed1 == TRUE)
    {
        Led_Lbl1->setPixmap(*pixmapGreenLed);
    }
    else
    {
        Led_Lbl1->setPixmap(*pixmapRedLed);
    }

    if(bLed2 == TRUE)
    {
        Led_Lbl2->setPixmap(*pixmapGreenLed);
    }
    else
    {
        Led_Lbl2->setPixmap(*pixmapRedLed);
    }

    if(bLed3 == TRUE)
    {
        Led_Lbl3->setPixmap(*pixmapGreenLed);
    }
    else
    {
        Led_Lbl3->setPixmap(*pixmapRedLed);
    }

    if(bLed4 == TRUE)
    {
        Led_Lbl4->setPixmap(*pixmapGreenLed);
    }
    else
    {
        Led_Lbl4->setPixmap(*pixmapRedLed);
    }

    //printf("bXCBRSwFlg: %d, bErthSwFlg: %d \n",bUpdtXCBRSwFlg,bUpdtErthSwFlg);

    if(bUpdtXCBRSwFlg == TRUE && bUpdtErthSwFlg == FALSE)
    {
       scene->addPixmap(pixmapESWOpenXCBRClose);
    }
    else if(bUpdtXCBRSwFlg == FALSE && bUpdtErthSwFlg == TRUE)
    {
       scene->addPixmap(pixmapESWCloseXCBROpen);
    }
    else if(bUpdtXCBRSwFlg == FALSE && bUpdtErthSwFlg == FALSE)
    {
        scene->addPixmap(pixmapESWOpenXCBROpen);
    }

    if(iAlarmFlg)
    {
        timerAlarmColor->start(400);
        //printf("iAlarmFlg: %d\n",iAlarmFlg);
    }
    else
    {
        timerAlarmColor->stop();
        Led_LblAlrm->setPixmap(*pixmapGreyLed);
        //printf("iAlarmFlg: stop\n");
    }
}

/*****************************************************************************
* pshbtnCloseHandler() Function
******************************************************************************
* This fucntion is handler for Close/Exit button. This function is called when
* Close/Exit button is clicked, it stops PIS10 stack and frees it.
******************************************************************************/
void QtClientWidget ::pshbtnCloseHandler()
{
    enum IEC61850_ErrorCodes eErrorCode = IEC61850_ERROR_NONE;

    eErrorCode = IEC61850_Stop(PIS10Client::pThis->GetMyClient()); // Starts myServer

    if(eErrorCode != IEC61850_ERROR_NONE)
    {
        printf("   Can't start server: %s (%i)\n", IEC61850_ErrorString(eErrorCode), eErrorCode);
    }

    eErrorCode = IEC61850_Free(PIS10Client::pThis->GetMyClient());  //Frees myServerClient

    if(eErrorCode == IEC61850_ERROR_NONE)
    {
        //MyServer = NULL;
    }
    else
    {
        printf("   Free IEC61850 Object has failed: %s (%i)\n", IEC61850_ErrorString(eErrorCode), eErrorCode);
    }

    qApp->quit();
}

/*****************************************************************************
* AlarmBtnColorChange() Function
******************************************************************************
* This fucntion is handler for Alarm timer.
* If Alarm is set then it flashes the led.
******************************************************************************/
void QtClientWidget :: AlarmBtnColorChange()
{
    //printf("iToggle: %d\n",iToggle);
    if(iToggle == 0)
    {
        Led_LblAlrm->setPixmap(*pixmapOrangeLed);
        iToggle = 1;
    }
    else
    {
        Led_LblAlrm->setPixmap(*pixmapGreyLed);
        iToggle = 0;
    }

}

