/*****************************************************************************/
/*!	\file sysctype.h
 *	\brief SystemCORP definition file
 *	\par 		SystemCORP Embedded Technology Pty Ltd
 *
 *			5/50 William St.,
 *			Beckenham,
 *			Perth, WA 6107,
 *			Australia.
 *
 *			Phone	: +61 (8) 9258 4683
 *			Fax	: +61 (8) 9258 4673
 *			Email	: info@systemcorp.com.au
 */
/*****************************************************************************/
#ifndef SYSCTYPE_H
#define SYSCTYPE_H 1


#if defined(VXWORX_ACE)
	#include "vxworks_ace.h"
#endif


	/*!\ Define TRUE and FALSE if not defined */
	#ifndef FALSE
		#define FALSE	 	0
	#endif
	#ifndef TRUE
		#define TRUE 			1
	#endif

	/*!\def BYTES_128 128 bytes size*/
	#define BYTES_NONE 		0
	/*!\def BYTES_64 64 bytes size*/
	#define BYTES_64 		64
	/*!\def BYTES_128 128 bytes size*/
	#define BYTES_128 		128
	/*!\def BYTES_256 256 bytes size*/
	#define BYTES_256 		256
	/*!\def KILOBYTE 1024 bytes size*/
	#define KILOBYTE 		1024
	/*!\def KILOBYTES_2 2048 bytes size*/
	#define KILOBYTES_2 		2048 

	
#ifdef BORLAND_BOOLEAN
	/*!	\brief Size : 8 bits	Range : TRUE or FALSE */
	#define Boolean				bool
#else
	/*!	\brief Size : 8 bits	Range : TRUE or FALSE */
	typedef unsigned char 		Boolean;
#endif

//	typedef unsigned char 		Boolean;
#if  defined(WIN32) && (! defined(_NI_mswin32_) ) 
	/*!	\brief Size : 8 bits	Range : 0 to 255 */	
	typedef unsigned char 			Unsigned8;
	/*!	\brief Size : 8 bits	Range : -128 to 127 */	
	typedef signed char 			Integer8;
	/*!	\brief Size : 16 bits	Range : 0 to 65,535  */	
	typedef unsigned  short  int	Unsigned16;
	/*!	\brief Size : 16 bits	Range : -32,768 to 32,767 */	
	typedef signed short int 		Integer16;
	/*!	\brief Size : 32 bits	Range : 0 to 4,294,967,295 */	
	typedef unsigned int			Unsigned32;
	/*!	\brief Size : 32 bits	Range : -2,147,483,648 to 2,147,483,647 */	
	typedef signed int 				Integer32;
	/*!	\brief Size : 64 bits	Range : 0 to 18,446,744,073,709,551,615 */	
	typedef unsigned long long 		Unsigned64;
	/*!	\brief Size : 64 bits	Range : ?,223,372,036,854,775,808 to 9,223,372,036,854,775,807 */	
	typedef signed long long		Integer64;

#elif  defined(_NI_mswin32_) 
	/*!	\brief Size : 8 bits	Range : 0 to 255 */	
	typedef unsigned char 			Unsigned8;
	/*!	\brief Size : 8 bits	Range : -128 to 127 */	
	typedef signed char 			Integer8;
	/*!	\brief Size : 16 bits	Range : 0 to 65,535  */	
	typedef unsigned  short  int	Unsigned16;
	/*!	\brief Size : 16 bits	Range : -32,768 to 32,767 */	
	typedef signed short int 		Integer16;
	/*!	\brief Size : 32 bits	Range : 0 to 4,294,967,295 */	
	typedef unsigned int			Unsigned32;
	/*!	\brief Size : 32 bits	Range : -2,147,483,648 to 2,147,483,647 */	
	typedef signed int 				Integer32;
	/*!	\brief Size : 64 bits	Range : 0 to 18,446,744,073,709,551,615 */	
	typedef unsigned __int64 		Unsigned64;
	/*!	\brief Size : 64 bits	Range : 9,223,372,036,854,775,808 to 9,223,372,036,854,775,807 */	
	typedef signed __int64			Integer64;
#elif defined(__linux__) 
	#include	<stdint.h>
	/*!	\brief Size : 8 bits	Range : 0 to 255 */	
	typedef uint8_t 				Unsigned8;
	/*!	\brief Size : 8 bits	Range : -128 to 127 */	
	typedef int8_t 				Integer8;
	/*!	\brief Size : 16 bits	Range : 0 to 65,535  */	
	typedef uint16_t				Unsigned16;
	/*!	\brief Size : 16 bits	Range : -32,768 to 32,767 */	
	typedef int16_t 				Integer16;
	/*!	\brief Size : 32 bits	Range : 0 to 4,294,967,295 */	
	typedef uint32_t				Unsigned32;
	/*!	\brief Size : 32 bits	Range : -2,147,483,648 to 2,147,483,647 */	
	typedef int32_t 				Integer32;
	/*!	\brief Size : 64 bits	Range : 0 to 18,446,744,073,709,551,615 */	
	typedef uint64_t				Unsigned64;
	/*!	\brief Size : 64 bits	Range : ?,223,372,036,854,775,808 to 9,223,372,036,854,775,807 */	
	typedef int64_t 				Integer64;
#elif defined(VXWORX_ACE)
	/*!	\brief Size : 8 bits	Range : 0 to 255 */	
	typedef unsigned char	 		Unsigned8;
	/*!	\brief Size : 8 bits	Range : -128 to 127 */	
	typedef signed char				Integer8;
	/*!	\brief Size : 16 bits	Range : 0 to 65,535  */	
	typedef unsigned short			Unsigned16;
	/*!	\brief Size : 16 bits	Range : -32,768 to 32,767 */	
	typedef signed short			Integer16;
	/*!	\brief Size : 32 bits	Range : 0 to 4,294,967,295 */	
	typedef unsigned int			Unsigned32;
	/*!	\brief Size : 32 bits	Range : -2,147,483,648 to 2,147,483,647 */	
	typedef signed int 				Integer32;
	/*!	\brief Size : 64 bits	Range : 0 to 18,446,744,073,709,551,615 */	
	typedef unsigned long long		Unsigned64;
	/*!	\brief Size : 64 bits	Range : ?,223,372,036,854,775,808 to 9,223,372,036,854,775,807 */	
	typedef signed long long		Integer64;
#else
	/*!	\brief Size : 8 bits	Range : 0 to 255 */	
	typedef unsigned char 			Unsigned8;
	/*!	\brief Size : 8 bits	Range : -128 to 127 */	
	typedef signed char 			Integer8;
	/*!	\brief Size : 16 bits	Range : 0 to 65,535  */	
	typedef unsigned   int			Unsigned16;
	/*!	\brief Size : 16 bits	Range : -32,768 to 32,767 */	
	typedef signed  int 			Integer16;
	/*!	\brief Size : 32 bits	Range : 0 to 4,294,967,295 */	
	typedef unsigned long int		Unsigned32;
	/*!	\brief Size : 32 bits	Range : -2,147,483,648 to 2,147,483,647 */	
	typedef signed long int 			Integer32;
#endif
	/*!	\brief Size : 32 bits	Range : 1E-37 to 1E+37 with 6 digits of precision */	
	typedef float					Float32;
	/*!	\brief Size : 64 bits	Range : 1E-37 to 1E+37 with 10 digits of precision than Float32 */	
	typedef double				Float64;
	/*!	\brief Size : 80 bits	Range : 1E-37 to 1E+37 with 10 digits of precision than Float64 */	
	typedef long double			Float80;	
#endif

// End of File
